use std::fs::create_dir;

use helmzoo_lib::{
    cli::{copy_recursively, is_path_relative},
    output::message_info,
};
use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize, PartialEq, Debug, Clone)]
pub(crate) struct Extension {
    name: Option<String>,
    target_dir: String,
    pub(crate) source_dir: String,
}

impl Extension {
    pub(crate) fn apply(
        &self,
        chart_path: String,
        config_path: String,
    ) -> Result<(), Box<dyn std::error::Error>> {
        let extension_path = match is_path_relative(self.source_dir.clone()) {
            true => format!("{}/{}", config_path, self.source_dir),
            false => self.source_dir.clone(),
        };
        let extension_name = match self.name.clone() {
            Some(res) => res,
            None => "Unnamed".to_string(),
        };
        message_info(&format!("applying extension: '{}'", extension_name));
        let target_dir = format!("{}/{}", chart_path, self.target_dir);
        message_info(&format!("trying to create a dir: {}", target_dir));
        create_dir(target_dir.clone())?;
        message_info(&format!("copying {} to {}", extension_path, target_dir));
        copy_recursively(extension_path, target_dir)?;
        Ok(())
    }
}
