use serde::{Deserialize, Serialize};

use crate::config::ChartExtended;

pub(crate) mod custom_command;
pub(crate) mod git;

pub(crate) trait Target {
    fn push(
        &self,
        workdir_path: String,
        chart_path: String,
        chart_local: ChartExtended,
        dry_run: bool,
    ) -> Result<(), Box<dyn std::error::Error>>;
}

#[derive(Serialize, Deserialize, PartialEq, Debug, Clone)]
pub(crate) struct Mirror {
    pub(crate) name: String,
    pub(crate) git: Option<git::GitMirror>,
    pub(crate) custom_command: Option<CustomCommandsMirror>,
}

#[derive(Serialize, Deserialize, PartialEq, Debug, Clone)]
pub(crate) struct CustomCommandsMirror {
    pub(crate) package: Vec<String>,
    pub(crate) upload: Vec<String>,
}

pub(crate) fn mirror_from_mirror_obj(
    mirror: Mirror,
) -> Result<Box<dyn Target>, Box<dyn std::error::Error>> {
    if let Some(git) = mirror.git {
        return Ok(Box::from(git));
    } else if let Some(command) = mirror.custom_command {
        return Ok(Box::from(custom_command::CustomCommands {
            package: command.package,
            upload: command.upload,
        }));
    }
    Err(Box::from(format!(
        "a kind is unknown for the mirror {}",
        mirror.name
    )))
}
